import argparse
import paths as pt
from src.data import parse_and_clean_data, make_screenings
from src.data import make_clusters, make_dataset_full
from src.data import make_dataset_count, make_dataset_emb
from src.model import make_xgb_models

def str2bool(v):
    if isinstance(v, bool):
       return v
    if v.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    else:
        raise argparse.ArgumentTypeError('Boolean value expected.')

def parse_arguments():
    parser = argparse.ArgumentParser(description='A client for AIR')
    parser.add_argument('--dataset-year', type=str, default="2020",
                        choices=['2019', '2020'], help='string indicating dataset year')
    parser.add_argument('--dataset-version', type=str, default="emb",
                        choices=['emb', 'ohe'], help='string indicating dataset version')
    parser.add_argument("--enable-visualization", type=str2bool, nargs='?',
                        const=True, default=False,
                        help="boolean indicating if visualization should be enabled.")
    parser.add_argument("--use-real-ats-names", type=str2bool, nargs='?',
                        const=True, default=False,
                        help="boolean indicating if we should use real ats names.")
    parser.add_argument("--run-full-pipeline", type=str2bool, nargs='?',
                        const=True, default=True,
                        help="boolean indicating if we should run full pipeline. " +
                        "set to false to only make models")
    return parser.parse_args()

def main():
    parsed_args = parse_arguments()
    dataset_year = parsed_args.dataset_year
    dataset_version = parsed_args.dataset_version
    enable_visualization = parsed_args.enable_visualization
    use_real_ats_names = parsed_args.use_real_ats_names
    run_full_pipeline = parsed_args.run_full_pipeline
    print(f"Client started. Using this configuration:")
    print(f"Raw data dictionary: {pt.RAW_DATA_DIR_2020}")
    print(f"Dataset year: {dataset_year}")
    print(f"Dataset version: {dataset_version}")
    print(f"Visualization enabled: {enable_visualization}")
    print(f"Use real ATS names: {use_real_ats_names}")
    print(f"Run full pipeline: {run_full_pipeline}\n")
    
    if run_full_pipeline:
        print("Now parsing and cleaning data ...")        
        if dataset_year == '2019':
            parse_and_clean_data.main(year=dataset_year)
        else:
            parse_and_clean_data.main()
        
        print("Extracting screenings ...")
        make_screenings.main()
    
        print("Making clusters ...")
        make_clusters.main()
        print(f"Completed making cluster model. It can be found at: {pt.CLUSTERS_DIR}\n")
    
        print("Making full dataset ...")
        make_dataset_full.main(use_real_ats_names)
    
        if dataset_version == "emb":
            print("Making dataset with embedded ats ...")
            make_dataset_emb.main(enable_visualization)
        else:
            print("Making dataset with one-hot encoded ats ...")
            make_dataset_count.main()
            
        print("\nCompleted generating datasets at:")
        print(f"Interim data dictionary: {pt.INTERIM_DATA_DIR}")
        print(f"Processed data dictionary: {pt.PROCESSED_DATA_DIR}\n")
    
    print(f"Making 4 XGBoost models based on version: {dataset_version} ...\n")
    make_xgb_models.main(dataset_version)
    
    print(f"Completed making models. Models and SHAP plots can be found at:\n" +
          f"{pt.COMPLETE_XGB_DIR}\n" + f"{pt.COMPLIANCE_XGB_DIR}\n" +
          f"{pt.FALL_XGB_DIR}\n" + f"{pt.RISK_XGB_DIR}" + "\n")
    
if __name__ == "__main__":
    main()